@@ -1,13 +1,8 @@
 from functools import lru_cache
 
 from django.conf import settings
-
-# Always import this module as follows:
-# from debug_toolbar import settings [as dt_settings]
-
-# Don't import directly CONFIG or PANELs, or you will miss changes performed
-# with override_settings in tests.
-
+from django.dispatch import receiver
+from django.test.signals import setting_changed
 
 CONFIG_DEFAULTS = {
     # Toolbar options
@@ -17,13 +12,14 @@ CONFIG_DEFAULTS = {
     },
     "INSERT_BEFORE": "</body>",
     "RENDER_PANELS": None,
-    "RESULTS_CACHE_SIZE": 10,
+    "RESULTS_CACHE_SIZE": 25,
     "ROOT_TAG_EXTRA_ATTRS": "",
     "SHOW_COLLAPSED": False,
     "SHOW_TOOLBAR_CALLBACK": "debug_toolbar.middleware.show_toolbar",
     # Panel options
     "EXTRA_SIGNALS": [],
     "ENABLE_STACKTRACES": True,
+    "ENABLE_STACKTRACES_LOCALS": False,
     "HIDE_IN_STACKTRACES": (
         "socketserver",
         "threading",
@@ -36,10 +32,12 @@ CONFIG_DEFAULTS = {
         "django.utils.deprecation",
         "django.utils.functional",
     ),
+    "PRETTIFY_SQL": True,
     "PROFILER_MAX_DEPTH": 10,
     "SHOW_TEMPLATE_CONTEXT": True,
     "SKIP_TEMPLATE_PREFIXES": ("django/forms/widgets/", "admin/widgets/"),
     "SQL_WARNING_THRESHOLD": 500,  # milliseconds
+    "OBSERVE_REQUEST_CALLBACK": "debug_toolbar.toolbar.observe_request",
 }
 
 
@@ -52,6 +50,7 @@ def get_config():
 
 
 PANELS_DEFAULTS = [
+    "debug_toolbar.panels.history.HistoryPanel",
     "debug_toolbar.panels.versions.VersionsPanel",
     "debug_toolbar.panels.timer.TimerPanel",
     "debug_toolbar.panels.settings.SettingsPanel",
@@ -75,3 +74,18 @@ def get_panels():
     except AttributeError:
         PANELS = PANELS_DEFAULTS
     return PANELS
+
+
+@receiver(setting_changed)
+def update_toolbar_config(*, setting, **kwargs):
+    """
+    Refresh configuration when overriding settings.
+    """
+    if setting == "DEBUG_TOOLBAR_CONFIG":
+        get_config.cache_clear()
+    elif setting == "DEBUG_TOOLBAR_PANELS":
+        from debug_toolbar.toolbar import DebugToolbar
+
+        get_panels.cache_clear()
+        DebugToolbar._panel_classes = None
+        # Not implemented: invalidate debug_toolbar.urls.
