@@ -2,7 +2,7 @@ from django.contrib import admin
 from django.urls import include, path
 from django.views.generic import TemplateView
 
-import debug_toolbar
+from example.views import increment
 
 urlpatterns = [
     path("", TemplateView.as_view(template_name="index.html")),
@@ -10,5 +10,6 @@ urlpatterns = [
     path("mootools/", TemplateView.as_view(template_name="mootools/index.html")),
     path("prototype/", TemplateView.as_view(template_name="prototype/index.html")),
     path("admin/", admin.site.urls),
-    path("__debug__/", include(debug_toolbar.urls)),
+    path("ajax/increment", increment, name="ajax_increment"),
+    path("__debug__/", include("debug_toolbar.urls")),
 ]
