@@ -1,11 +1,13 @@
+import asyncio
 import datetime
 import os
 import unittest
 from unittest.mock import patch
 
 import django
+from asgiref.sync import sync_to_async
 from django.contrib.auth.models import User
-from django.db import connection
+from django.db import connection, transaction
 from django.db.models import Count
 from django.db.utils import DatabaseError
 from django.shortcuts import render
@@ -14,17 +16,15 @@ from django.test.utils import override_settings
 import debug_toolbar.panels.sql.tracking as sql_tracking
 from debug_toolbar import settings as dt_settings
 
-from ..base import BaseTestCase
+from ..base import BaseMultiDBTestCase, BaseTestCase
+from ..models import PostgresJSON
 
-try:
-    from psycopg2._json import Json as PostgresJson
-except ImportError:
-    PostgresJson = None
 
-if connection.vendor == "postgresql":
-    from ..models import PostgresJSON as PostgresJSONModel
-else:
-    PostgresJSONModel = None
+def sql_call(use_iterator=False):
+    qs = User.objects.all()
+    if use_iterator:
+        qs = qs.iterator()
+    return list(qs)
 
 
 class SQLPanelTestCase(BaseTestCase):
@@ -39,18 +39,18 @@ class SQLPanelTestCase(BaseTestCase):
     def test_recording(self):
         self.assertEqual(len(self.panel._queries), 0)
 
-        list(User.objects.all())
+        sql_call()
 
         # ensure query was logged
         self.assertEqual(len(self.panel._queries), 1)
         query = self.panel._queries[0]
-        self.assertEqual(query[0], "default")
-        self.assertTrue("sql" in query[1])
-        self.assertTrue("duration" in query[1])
-        self.assertTrue("stacktrace" in query[1])
+        self.assertEqual(query["alias"], "default")
+        self.assertTrue("sql" in query)
+        self.assertTrue("duration" in query)
+        self.assertTrue("stacktrace" in query)
 
         # ensure the stacktrace is populated
-        self.assertTrue(len(query[1]["stacktrace"]) > 0)
+        self.assertTrue(len(query["stacktrace"]) > 0)
 
     @unittest.skipUnless(
         connection.vendor == "postgresql", "Test valid only on PostgreSQL"
@@ -58,29 +58,66 @@ class SQLPanelTestCase(BaseTestCase):
     def test_recording_chunked_cursor(self):
         self.assertEqual(len(self.panel._queries), 0)
 
-        list(User.objects.all().iterator())
+        sql_call(use_iterator=True)
 
         # ensure query was logged
         self.assertEqual(len(self.panel._queries), 1)
 
-    @patch("debug_toolbar.panels.sql.tracking.state", wraps=sql_tracking.state)
-    def test_cursor_wrapper_singleton(self, mock_state):
-        list(User.objects.all())
+    @patch(
+        "debug_toolbar.panels.sql.tracking.NormalCursorWrapper",
+        wraps=sql_tracking.NormalCursorWrapper,
+    )
+    def test_cursor_wrapper_singleton(self, mock_wrapper):
+        sql_call()
 
         # ensure that cursor wrapping is applied only once
-        self.assertEqual(mock_state.Wrapper.call_count, 1)
+        self.assertEqual(mock_wrapper.call_count, 1)
 
-    @patch("debug_toolbar.panels.sql.tracking.state", wraps=sql_tracking.state)
-    def test_chunked_cursor_wrapper_singleton(self, mock_state):
-        list(User.objects.all().iterator())
+    @patch(
+        "debug_toolbar.panels.sql.tracking.NormalCursorWrapper",
+        wraps=sql_tracking.NormalCursorWrapper,
+    )
+    def test_chunked_cursor_wrapper_singleton(self, mock_wrapper):
+        sql_call(use_iterator=True)
 
         # ensure that cursor wrapping is applied only once
-        self.assertEqual(mock_state.Wrapper.call_count, 1)
+        self.assertEqual(mock_wrapper.call_count, 1)
+
+    @patch(
+        "debug_toolbar.panels.sql.tracking.NormalCursorWrapper",
+        wraps=sql_tracking.NormalCursorWrapper,
+    )
+    async def test_cursor_wrapper_async(self, mock_wrapper):
+        await sync_to_async(sql_call)()
+
+        self.assertEqual(mock_wrapper.call_count, 1)
+
+    @patch(
+        "debug_toolbar.panels.sql.tracking.NormalCursorWrapper",
+        wraps=sql_tracking.NormalCursorWrapper,
+    )
+    async def test_cursor_wrapper_asyncio_ctx(self, mock_wrapper):
+        self.assertTrue(sql_tracking.allow_sql.get())
+        await sync_to_async(sql_call)()
+
+        async def task():
+            sql_tracking.allow_sql.set(False)
+            # By disabling sql_tracking.allow_sql, we are indicating that any
+            # future SQL queries should be stopped. If SQL query occurs,
+            # it raises an exception.
+            with self.assertRaises(sql_tracking.SQLQueryTriggered):
+                await sync_to_async(sql_call)()
+
+        # Ensure this is called in another context
+        await asyncio.create_task(task())
+        # Because it was called in another context, it should not have affected ours
+        self.assertTrue(sql_tracking.allow_sql.get())
+        self.assertEqual(mock_wrapper.call_count, 1)
 
     def test_generate_server_timing(self):
         self.assertEqual(len(self.panel._queries), 0)
 
-        list(User.objects.all())
+        sql_call()
 
         response = self.panel.process_request(self.request)
         self.panel.generate_stats(self.request, response)
@@ -91,7 +128,7 @@ class SQLPanelTestCase(BaseTestCase):
         query = self.panel._queries[0]
 
         expected_data = {
-            "sql_time": {"title": "SQL 1 queries", "value": query[1]["duration"]}
+            "sql_time": {"title": "SQL 1 queries", "value": query["duration"]}
         }
 
         self.assertEqual(self.panel.get_server_timing_stats(), expected_data)
@@ -130,7 +167,13 @@ class SQLPanelTestCase(BaseTestCase):
             .filter(group_count__lt=10)
             .filter(group_count__gt=1)
         )
-        list(User.objects.filter(date_joined=datetime.datetime(2017, 12, 22, 16, 7, 1)))
+        list(
+            User.objects.filter(
+                date_joined=datetime.datetime(
+                    2017, 12, 22, 16, 7, 1, tzinfo=datetime.timezone.utc
+                )
+            )
+        )
 
         response = self.panel.process_request(self.request)
         self.panel.generate_stats(self.request, response)
@@ -138,16 +181,27 @@ class SQLPanelTestCase(BaseTestCase):
         # ensure query was logged
         self.assertEqual(len(self.panel._queries), 3)
 
-        if django.VERSION >= (3, 1):
-            self.assertEqual(
-                tuple([q[1]["params"] for q in self.panel._queries]),
-                ('["Foo"]', "[10, 1]", '["2017-12-22 16:07:01"]'),
-            )
+        if connection.vendor == "mysql" and django.VERSION >= (4, 1):
+            # Django 4.1 started passing true/false back for boolean
+            # comparisons in MySQL.
+            expected_bools = '["Foo", true, false]'
         else:
-            self.assertEqual(
-                tuple([q[1]["params"] for q in self.panel._queries]),
-                ('["Foo", true, false]', "[10, 1]", '["2017-12-22 16:07:01"]'),
-            )
+            expected_bools = '["Foo"]'
+
+        if connection.vendor == "postgresql":
+            # PostgreSQL always includes timezone
+            expected_datetime = '["2017-12-22 16:07:01+00:00"]'
+        else:
+            expected_datetime = '["2017-12-22 16:07:01"]'
+
+        self.assertEqual(
+            tuple(query["params"] for query in self.panel._queries),
+            (
+                expected_bools,
+                "[10, 1]",
+                expected_datetime,
+            ),
+        )
 
     @unittest.skipUnless(
         connection.vendor == "postgresql", "Test valid only on PostgreSQL"
@@ -155,7 +209,7 @@ class SQLPanelTestCase(BaseTestCase):
     def test_json_param_conversion(self):
         self.assertEqual(len(self.panel._queries), 0)
 
-        list(PostgresJSONModel.objects.filter(field__contains={"foo": "bar"}))
+        list(PostgresJSON.objects.filter(field__contains={"foo": "bar"}))
 
         response = self.panel.process_request(self.request)
         self.panel.generate_stats(self.request, response)
@@ -163,14 +217,9 @@ class SQLPanelTestCase(BaseTestCase):
         # ensure query was logged
         self.assertEqual(len(self.panel._queries), 1)
         self.assertEqual(
-            self.panel._queries[0][1]["params"],
+            self.panel._queries[0]["params"],
             '["{\\"foo\\": \\"bar\\"}"]',
         )
-        if django.VERSION < (3, 1):
-            self.assertIsInstance(
-                self.panel._queries[0][1]["raw_params"][0],
-                PostgresJson,
-            )
 
     def test_binary_param_force_text(self):
         self.assertEqual(len(self.panel._queries), 0)
@@ -188,7 +237,7 @@ class SQLPanelTestCase(BaseTestCase):
         self.assertIn(
             "<strong>SELECT</strong> * <strong>FROM</strong>"
             " tests_binary <strong>WHERE</strong> field =",
-            self.panel._queries[0][1]["sql"],
+            self.panel._queries[0]["sql"],
         )
 
     @unittest.skipUnless(connection.vendor != "sqlite", "Test invalid for SQLite")
@@ -239,7 +288,7 @@ class SQLPanelTestCase(BaseTestCase):
         self.assertEqual(len(self.panel._queries), 2)
 
         self.assertEqual(
-            tuple([q[1]["params"] for q in self.panel._queries]),
+            tuple(query["params"] for query in self.panel._queries),
             (
                 '["Foo", true, false, "2017-12-22 16:07:01"]',
                 " ".join(
@@ -258,7 +307,7 @@ class SQLPanelTestCase(BaseTestCase):
         Test that the panel only inserts content after generate_stats and
         not the process_request.
         """
-        list(User.objects.filter(username="café".encode("utf-8")))
+        list(User.objects.filter(username="café".encode()))
         response = self.panel.process_request(self.request)
         # ensure the panel does not have content yet.
         self.assertNotIn("café", self.panel.content)
@@ -273,8 +322,8 @@ class SQLPanelTestCase(BaseTestCase):
         """
         Test that the panel inserts locals() content.
         """
-        local_var = "<script>alert('test');</script>"  # noqa
-        list(User.objects.filter(username="café".encode("utf-8")))
+        local_var = "<script>alert('test');</script>"  # noqa: F841
+        list(User.objects.filter(username="café".encode()))
         response = self.panel.process_request(self.request)
         self.panel.generate_stats(self.request, response)
         self.assertIn("local_var", self.panel.content)
@@ -288,7 +337,7 @@ class SQLPanelTestCase(BaseTestCase):
         """
         Test that the panel does not insert locals() content.
         """
-        list(User.objects.filter(username="café".encode("utf-8")))
+        list(User.objects.filter(username="café".encode()))
         response = self.panel.process_request(self.request)
         self.panel.generate_stats(self.request, response)
         self.assertNotIn("djdt-locals", self.panel.content)
@@ -326,26 +375,26 @@ class SQLPanelTestCase(BaseTestCase):
         self.assertEqual(len(self.panel._queries), 1)
 
         query = self.panel._queries[0]
-        self.assertEqual(query[0], "default")
-        self.assertTrue("sql" in query[1])
-        self.assertEqual(query[1]["sql"], 'select "username" from "auth_user"')
+        self.assertEqual(query["alias"], "default")
+        self.assertTrue("sql" in query)
+        self.assertEqual(query["sql"], 'select "username" from "auth_user"')
 
     def test_disable_stacktraces(self):
         self.assertEqual(len(self.panel._queries), 0)
 
         with self.settings(DEBUG_TOOLBAR_CONFIG={"ENABLE_STACKTRACES": False}):
-            list(User.objects.all())
+            sql_call()
 
         # ensure query was logged
         self.assertEqual(len(self.panel._queries), 1)
         query = self.panel._queries[0]
-        self.assertEqual(query[0], "default")
-        self.assertTrue("sql" in query[1])
-        self.assertTrue("duration" in query[1])
-        self.assertTrue("stacktrace" in query[1])
+        self.assertEqual(query["alias"], "default")
+        self.assertTrue("sql" in query)
+        self.assertTrue("duration" in query)
+        self.assertTrue("stacktrace" in query)
 
         # ensure the stacktrace is empty
-        self.assertEqual([], query[1]["stacktrace"])
+        self.assertEqual([], query["stacktrace"])
 
     @override_settings(
         DEBUG=True,
@@ -369,13 +418,13 @@ class SQLPanelTestCase(BaseTestCase):
         # template is loaded and basic.html extends base.html.
         self.assertEqual(len(self.panel._queries), 2)
         query = self.panel._queries[0]
-        self.assertEqual(query[0], "default")
-        self.assertTrue("sql" in query[1])
-        self.assertTrue("duration" in query[1])
-        self.assertTrue("stacktrace" in query[1])
+        self.assertEqual(query["alias"], "default")
+        self.assertTrue("sql" in query)
+        self.assertTrue("duration" in query)
+        self.assertTrue("stacktrace" in query)
 
         # ensure the stacktrace is populated
-        self.assertTrue(len(query[1]["stacktrace"]) > 0)
+        self.assertTrue(len(query["stacktrace"]) > 0)
 
     @override_settings(
         DEBUG_TOOLBAR_CONFIG={"PRETTIFY_SQL": True},
@@ -390,7 +439,7 @@ class SQLPanelTestCase(BaseTestCase):
 
         response = self.panel.process_request(self.request)
         self.panel.generate_stats(self.request, response)
-        pretty_sql = self.panel._queries[-1][1]["sql"]
+        pretty_sql = self.panel._queries[-1]["sql"]
         self.assertEqual(len(self.panel._queries), 1)
 
         # Reset the queries
@@ -401,7 +450,7 @@ class SQLPanelTestCase(BaseTestCase):
         response = self.panel.process_request(self.request)
         self.panel.generate_stats(self.request, response)
         self.assertEqual(len(self.panel._queries), 1)
-        self.assertNotEqual(pretty_sql, self.panel._queries[-1][1]["sql"])
+        self.assertNotEqual(pretty_sql, self.panel._queries[-1]["sql"])
 
         self.panel._queries = []
         # Run it again, but with prettyify back on.
@@ -412,7 +461,7 @@ class SQLPanelTestCase(BaseTestCase):
         response = self.panel.process_request(self.request)
         self.panel.generate_stats(self.request, response)
         self.assertEqual(len(self.panel._queries), 1)
-        self.assertEqual(pretty_sql, self.panel._queries[-1][1]["sql"])
+        self.assertEqual(pretty_sql, self.panel._queries[-1]["sql"])
 
     @override_settings(
         DEBUG=True,
@@ -430,7 +479,7 @@ class SQLPanelTestCase(BaseTestCase):
         self.assertEqual(len(self.panel._queries), 1)
 
         query = self.panel._queries[0]
-        template_info = query[1]["template_info"]
+        template_info = query["template_info"]
         template_name = os.path.basename(template_info["name"])
         self.assertEqual(template_name, "flat.html")
         self.assertEqual(template_info["context"][2]["content"].strip(), "{{ users }}")
@@ -452,8 +501,160 @@ class SQLPanelTestCase(BaseTestCase):
         self.assertEqual(len(self.panel._queries), 1)
 
         query = self.panel._queries[0]
-        template_info = query[1]["template_info"]
+        template_info = query["template_info"]
         template_name = os.path.basename(template_info["name"])
         self.assertEqual(template_name, "included.html")
         self.assertEqual(template_info["context"][0]["content"].strip(), "{{ users }}")
         self.assertEqual(template_info["context"][0]["highlight"], True)
+
+    def test_similar_and_duplicate_grouping(self):
+        self.assertEqual(len(self.panel._queries), 0)
+
+        User.objects.filter(id=1).count()
+        User.objects.filter(id=1).count()
+        User.objects.filter(id=2).count()
+        User.objects.filter(id__lt=10).count()
+        User.objects.filter(id__lt=20).count()
+        User.objects.filter(id__gt=10, id__lt=20).count()
+
+        response = self.panel.process_request(self.request)
+        self.panel.generate_stats(self.request, response)
+
+        self.assertEqual(len(self.panel._queries), 6)
+
+        queries = self.panel._queries
+        query = queries[0]
+        self.assertEqual(query["similar_count"], 3)
+        self.assertEqual(query["duplicate_count"], 2)
+
+        query = queries[1]
+        self.assertEqual(query["similar_count"], 3)
+        self.assertEqual(query["duplicate_count"], 2)
+
+        query = queries[2]
+        self.assertEqual(query["similar_count"], 3)
+        self.assertTrue("duplicate_count" not in query)
+
+        query = queries[3]
+        self.assertEqual(query["similar_count"], 2)
+        self.assertTrue("duplicate_count" not in query)
+
+        query = queries[4]
+        self.assertEqual(query["similar_count"], 2)
+        self.assertTrue("duplicate_count" not in query)
+
+        query = queries[5]
+        self.assertTrue("similar_count" not in query)
+        self.assertTrue("duplicate_count" not in query)
+
+        self.assertEqual(queries[0]["similar_color"], queries[1]["similar_color"])
+        self.assertEqual(queries[0]["similar_color"], queries[2]["similar_color"])
+        self.assertEqual(queries[0]["duplicate_color"], queries[1]["duplicate_color"])
+        self.assertNotEqual(queries[0]["similar_color"], queries[0]["duplicate_color"])
+
+        self.assertEqual(queries[3]["similar_color"], queries[4]["similar_color"])
+        self.assertNotEqual(queries[0]["similar_color"], queries[3]["similar_color"])
+        self.assertNotEqual(queries[0]["duplicate_color"], queries[3]["similar_color"])
+
+
+class SQLPanelMultiDBTestCase(BaseMultiDBTestCase):
+    panel_id = "SQLPanel"
+
+    def test_aliases(self):
+        self.assertFalse(self.panel._queries)
+
+        list(User.objects.all())
+        list(User.objects.using("replica").all())
+
+        response = self.panel.process_request(self.request)
+        self.panel.generate_stats(self.request, response)
+
+        self.assertTrue(self.panel._queries)
+
+        query = self.panel._queries[0]
+        self.assertEqual(query["alias"], "default")
+
+        query = self.panel._queries[-1]
+        self.assertEqual(query["alias"], "replica")
+
+    def test_transaction_status(self):
+        """
+        Test case for tracking the transaction status is properly associated with
+        queries on PostgreSQL, and that transactions aren't broken on other database
+        engines.
+        """
+        self.assertEqual(len(self.panel._queries), 0)
+
+        with transaction.atomic():
+            list(User.objects.all())
+            list(User.objects.using("replica").all())
+
+            with transaction.atomic(using="replica"):
+                list(User.objects.all())
+                list(User.objects.using("replica").all())
+
+        with transaction.atomic():
+            list(User.objects.all())
+
+        list(User.objects.using("replica").all())
+
+        response = self.panel.process_request(self.request)
+        self.panel.generate_stats(self.request, response)
+
+        if connection.vendor == "postgresql":
+            # Connection tracking is currently only implemented for PostgreSQL.
+            self.assertEqual(len(self.panel._queries), 6)
+
+            query = self.panel._queries[0]
+            self.assertEqual(query["alias"], "default")
+            self.assertIsNotNone(query["trans_id"])
+            self.assertTrue(query["starts_trans"])
+            self.assertTrue(query["in_trans"])
+            self.assertFalse("end_trans" in query)
+
+            query = self.panel._queries[-1]
+            self.assertEqual(query["alias"], "replica")
+            self.assertIsNone(query["trans_id"])
+            self.assertFalse("starts_trans" in query)
+            self.assertFalse("in_trans" in query)
+            self.assertFalse("end_trans" in query)
+
+            query = self.panel._queries[2]
+            self.assertEqual(query["alias"], "default")
+            self.assertIsNotNone(query["trans_id"])
+            self.assertEqual(query["trans_id"], self.panel._queries[0]["trans_id"])
+            self.assertFalse("starts_trans" in query)
+            self.assertTrue(query["in_trans"])
+            self.assertTrue(query["ends_trans"])
+
+            query = self.panel._queries[3]
+            self.assertEqual(query["alias"], "replica")
+            self.assertIsNotNone(query["trans_id"])
+            self.assertNotEqual(query["trans_id"], self.panel._queries[0]["trans_id"])
+            self.assertTrue(query["starts_trans"])
+            self.assertTrue(query["in_trans"])
+            self.assertTrue(query["ends_trans"])
+
+            query = self.panel._queries[4]
+            self.assertEqual(query["alias"], "default")
+            self.assertIsNotNone(query["trans_id"])
+            self.assertNotEqual(query["trans_id"], self.panel._queries[0]["trans_id"])
+            self.assertNotEqual(query["trans_id"], self.panel._queries[3]["trans_id"])
+            self.assertTrue(query["starts_trans"])
+            self.assertTrue(query["in_trans"])
+            self.assertTrue(query["ends_trans"])
+
+            query = self.panel._queries[5]
+            self.assertEqual(query["alias"], "replica")
+            self.assertIsNone(query["trans_id"])
+            self.assertFalse("starts_trans" in query)
+            self.assertFalse("in_trans" in query)
+            self.assertFalse("end_trans" in query)
+        else:
+            # Ensure that nothing was recorded for other database engines.
+            self.assertTrue(self.panel._queries)
+            for query in self.panel._queries:
+                self.assertFalse("trans_id" in query)
+                self.assertFalse("starts_trans" in query)
+                self.assertFalse("in_trans" in query)
+                self.assertFalse("end_trans" in query)
