@@ -2,10 +2,10 @@ import os
 import re
 import unittest
 
-import django
 import html5lib
 from django.contrib.staticfiles.testing import StaticLiveServerTestCase
 from django.core import signing
+from django.core.cache import cache
 from django.db import connection
 from django.http import HttpResponse
 from django.template.loader import get_template
@@ -96,11 +96,53 @@ class DebugToolbarTestCase(BaseTestCase):
         # check toolbar insertion before "</body>"
         self.assertContains(response, "</div>\n</body>")
 
+    def test_middleware_no_injection_when_encoded(self):
+        def get_response(request):
+            response = HttpResponse("<html><body></body></html>")
+            response["Content-Encoding"] = "something"
+            return response
+
+        response = DebugToolbarMiddleware(get_response)(self.request)
+        self.assertEqual(response.content, b"<html><body></body></html>")
+
     def test_cache_page(self):
-        self.client.get("/cached_view/")
-        self.assertEqual(len(self.toolbar.get_panel_by_id("CachePanel").calls), 3)
-        self.client.get("/cached_view/")
-        self.assertEqual(len(self.toolbar.get_panel_by_id("CachePanel").calls), 5)
+        # Clear the cache before testing the views. Other tests that use cached_view
+        # may run earlier and cause fewer cache calls.
+        cache.clear()
+        response = self.client.get("/cached_view/")
+        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 3)
+        response = self.client.get("/cached_view/")
+        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 2)
+
+    @override_settings(ROOT_URLCONF="tests.urls_use_package_urls")
+    def test_include_package_urls(self):
+        """Test urlsconf that uses the debug_toolbar.urls in the include call"""
+        # Clear the cache before testing the views. Other tests that use cached_view
+        # may run earlier and cause fewer cache calls.
+        cache.clear()
+        response = self.client.get("/cached_view/")
+        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 3)
+        response = self.client.get("/cached_view/")
+        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 2)
+
+    def test_low_level_cache_view(self):
+        """Test cases when low level caching API is used within a request."""
+        response = self.client.get("/cached_low_level_view/")
+        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 2)
+        response = self.client.get("/cached_low_level_view/")
+        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 1)
+
+    def test_cache_disable_instrumentation(self):
+        """
+        Verify that middleware cache usages before and after
+        DebugToolbarMiddleware are not counted.
+        """
+        self.assertIsNone(cache.set("UseCacheAfterToolbar.before", None))
+        self.assertIsNone(cache.set("UseCacheAfterToolbar.after", None))
+        response = self.client.get("/execute_sql/")
+        self.assertEqual(cache.get("UseCacheAfterToolbar.before"), 1)
+        self.assertEqual(cache.get("UseCacheAfterToolbar.after"), 1)
+        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 0)
 
     def test_is_toolbar_request(self):
         self.request.path = "/__debug__/render_panel/"
@@ -376,7 +418,7 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
         self.assertEqual(response.status_code, 200)
 
     @override_settings(DEBUG_TOOLBAR_CONFIG={"DISABLE_PANELS": set()})
-    def test_incercept_redirects(self):
+    def test_intcercept_redirects(self):
         response = self.client.get("/redirect/")
         self.assertEqual(response.status_code, 200)
         # Link to LOCATION header.
@@ -410,10 +452,7 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
         self.assertEqual(response.status_code, 200)
         # The key None (without quotes) exists in the list of template
         # variables.
-        if django.VERSION < (3, 0):
-            self.assertIn("None: &#39;&#39;", response.json()["content"])
-        else:
-            self.assertIn("None: &#x27;&#x27;", response.json()["content"])
+        self.assertIn("None: &#x27;&#x27;", response.json()["content"])
 
 
 @unittest.skipIf(webdriver is None, "selenium isn't installed")
