@@ -2,8 +2,6 @@ from django.contrib import admin
 from django.contrib.auth.views import LoginView
 from django.urls import include, path, re_path
 
-import debug_toolbar
-
 from . import views
 from .models import NonAsciiRepr
 
@@ -20,9 +18,10 @@ urlpatterns = [
     path("new_user/", views.new_user),
     path("execute_sql/", views.execute_sql),
     path("cached_view/", views.cached_view),
+    path("cached_low_level_view/", views.cached_low_level_view),
     path("json_view/", views.json_view),
     path("redirect/", views.redirect_view),
     path("login_without_redirect/", LoginView.as_view(redirect_field_name=None)),
     path("admin/", admin.site.urls),
-    path("__debug__/", include(debug_toolbar.urls)),
+    path("__debug__/", include("debug_toolbar.urls")),
 ]
